/* Copyright 2006 Siva Dirisala */
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.URL;
import java.util.regex.*;

import javax.swing.*;
import javax.swing.table.*;

public class Sudoku implements ActionListener {

  public Sudoku(SudokuModel sm) {
    this.sm = sm;
    this.stable = new SudokuTable(this);
  }

  public void actionPerformed(ActionEvent e) {
    Window win = FocusManager.getCurrentManager().getFocusedWindow();
    if(win != null)
      win.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
    try {
    if("TIMER".equals(e.getActionCommand())) {
      if(!timer.pause()) timer.start();
      sm.fireTableDataChanged();
    }
    else if("HINT".equals(e.getActionCommand())) {
      sm.solve(false);
      if(sm.isGameOver()) timer.stop();
    }
    else if("SOLVE".equals(e.getActionCommand())) {
      sm.solve();
      if(sm.isGameOver()) timer.stop();
    }
    else if("RESET".equals(e.getActionCommand())) {
      sm.reset();
      timer.restart();
    }
    else if("SAVE".equals(e.getActionCommand()))
      sm.save();
    else if("NEW".equals(e.getActionCommand())) {
      if(sm.newBoard()) {
        timer.restart();
        if(sm.getLevel() == 5) {
          timer.stop();
	  timer.setText("");
	}
      }
    }
    else if("LEVEL".equals(e.getActionCommand())) {
      int level = ((JComboBox)e.getSource()).getSelectedIndex()+1;
      sm.setLevel(level);
    }
    }
    finally {
      win.setCursor(null);
    }
  }

  public static void main(String args[]) throws Exception {
    UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
    int level = 1;
    SudokuModel sm = null;
    boolean startTimer = true;
    if(args.length > 0) {
      try {
        level = Integer.parseInt(args[0]);
        sm = SudokuModel.generateSudoku(level);
      }
      catch(Exception ex) {
        sm = new SudokuModel(args[0]);
      }
    }
    else {
      sm = new SudokuModel();
      startTimer = false;
    }
    sm.level = level;
    Sudoku sudoku = new Sudoku(sm);
    SudokuTable table = sudoku.stable;
    JFrame fr = new JFrame("SKu SuDoku (siva.dirisala@gmail.com)");
    fr.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    int size = table.getRowHeight(0)*sm.board.length;
    table.setPreferredSize(new Dimension(size,size));
    JPanel panel = new JPanel();
    panel.add(table,BorderLayout.CENTER);
    JPanel controls = new JPanel();
    controls.setLayout(new GridLayout(7,1));
    String[] levels = { "Easy", "Medium", "Hard", "Evil" , "Custom" };
    JComboBox cb = new JComboBox(levels);
    cb.setSelectedIndex(level-1);
    cb.addActionListener(sudoku);
    cb.setActionCommand("LEVEL");
    controls.add(cb);
    sudoku.timer = new Clock();
    sudoku.timer.addActionListener(sudoku);
    sudoku.timer.setActionCommand("TIMER");
    controls.add(sudoku.timer);
    JButton newButton = new JButton("New");
    newButton.addActionListener(sudoku);
    newButton.setActionCommand("NEW");
    controls.add(newButton);
    JButton reset = new JButton("Reset");
    reset.addActionListener(sudoku);
    reset.setActionCommand("RESET");
    controls.add(reset);
    JButton hint = new JButton("Hint");
    hint.addActionListener(sudoku);
    hint.setActionCommand("HINT");
    controls.add(hint);
    JButton solve = new JButton("Solve");
    solve.addActionListener(sudoku);
    solve.setActionCommand("SOLVE");
    controls.add(solve);
    JButton save = new JButton("Save");
    save.addActionListener(sudoku);
    save.setActionCommand("SAVE");
    controls.add(save);
    panel.add(controls);
    fr.add(panel);
    fr.pack();
    fr.setVisible(true);
    if(startTimer)
      sudoku.timer.start();
  }

  protected SudokuModel sm;
  protected SudokuTable stable;
  protected Clock timer;
}

class SudokuTable extends JTable {

  public SudokuTable(Sudoku sudoku) {
    super(sudoku.sm);
    this.sudoku = sudoku;
    setRowHeight(getRowHeight()*2);
    setFont(getFont().deriveFont(0.8f*getRowHeight()));
  }

  public Object getValueAt(int row,int column) {
    if(sudoku.timer.isPaused()) return 0;
    return super.getValueAt(row,column);
  }

  public void setValueAt(Object obj,int row,int column) {
    super.setValueAt(obj,row,column);
    if(!sudoku.timer.isRunning()) 
      sudoku.sm.setGenerated(row,column,obj!=null);
    if(sudoku.sm.isGameOver())
      sudoku.timer.stop();
  }

  public Component prepareRenderer(TableCellRenderer tcr,int row,int col) {
    Component c = super.prepareRenderer(tcr,row,col);
    if(sudoku.sm.isCellGenerated(row,col)) {
      c.setForeground(Color.blue);
    }
    else
      c.setForeground(Color.black);
    return c;
  }

  public Component prepareEditor(TableCellEditor tce,int row,int col) {
    Component c = super.prepareEditor(tce,row,col);
    c.setFont(getFont());
    return c;
  }

  public void paint(Graphics g) {
    if(sudoku.sm.isGameOver())
      setBackground(Color.green);
    else
      setBackground(Color.white);
    super.paint(g);
    int sbs = sudoku.sm.getSubBoardSize();
    for(int i=sbs;i<getRowCount();i+=sbs) {
      Rectangle top = getCellRect(0,i,false);
      Rectangle bottom = getCellRect(getRowCount()-1,i,false);
      g.drawLine(top.x,top.y,bottom.x,bottom.y+bottom.height);
    }
    for(int i=sbs;i<getRowCount();i+=sbs) {
      Rectangle top = getCellRect(i,0,false);
      Rectangle bottom = getCellRect(i,getColumnCount()-1,false);
      g.drawLine(top.x,top.y,bottom.x+bottom.width,bottom.y);
    }
  }

  protected Sudoku sudoku;
}
