/* Copyright 2006 Siva Dirisala */
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.URL;
import java.util.regex.*;

import javax.swing.*;
import javax.swing.table.*;

public class SudokuModel extends AbstractTableModel {

  public SudokuModel() {
    this(9);
  }

  public SudokuModel(String file) {
    this();
    try {
      BufferedReader br = new BufferedReader(new FileReader(file));
      String line = null;
      int i=0;
      while((line = br.readLine()) != null) {
	char[] chars = line.toCharArray();
	for(int j=0;j<chars.length;j++)
          if(chars[j] != ' ') {
            board[i][j] = new Integer((chars[j]-'0'));
	    generated[i][j] = true;
	  }
	i++;
      }
    }
    catch(Exception ex) {
    }
  }

  public SudokuModel(int size) {
    board = new Integer[size][size];
    generated = new boolean[size][size];
    subBoardSize = (int)Math.sqrt(size);
  }

  public SudokuModel(Integer[][] board,boolean[][] generated) {
    this.board = board;
    this.generated = generated;
    subBoardSize = (int)Math.sqrt(board.length);
  }

  public static SudokuModel generateSudoku(int level) {
    if(level == 5) {
      return new SudokuModel();
    }
    String url = "http://view.websudoku.com/?level="+level; 
    Pattern tablePattern = Pattern.compile("(?i)<TABLE.*CLASS=S0.*</TABLE>");
    Pattern cellPattern = Pattern.compile("(?i)<INPUT[^>]*>");
    Pattern valuePattern = Pattern.compile("VALUE=\"([0-9])\"");
    InputStream istr = null;
    Integer[][] board = new Integer[9][9];
    boolean[][] generated = new boolean[9][9];
    int r=0;
    int c=0;
    try {
      URL u = new URL(url);
      istr = u.openStream();
      BufferedReader br = new BufferedReader(new InputStreamReader(istr));
      String line;
      while((line = br.readLine()) != null) {
        if(tablePattern.matcher(line).find()) {
          Matcher m = cellPattern.matcher(line);
          while(m.find()) {
            Matcher vm = valuePattern.matcher(m.group());
            if(vm.find()) {
              board[r][c] = new Integer(vm.group(1));
              generated[r][c] = true;
            }
            c++;
            if(c==9) {
              r++;
              c=0;
            }         
          }
          return new SudokuModel(board,generated); 
        }
      }
    }
    catch(Exception ex) {
      System.out.println(ex);
    }
    return null;
  }

  public void setLevel(int level) {
    this.level = level;
  }

  public int getLevel() {
    return level;
  }

  public int getRowCount() {
    return board.length;
  }

  public int getColumnCount() {
    return board.length;
  }

  public Class getColumnClass(int col) {
    return Integer.class;
  }

  public Object getValueAt(int row,int column) {
    return board[row][column]; 
  }

  public void setValueAt(Object value,int row,int column) {
    if(value == null || accept(row,column,((Integer)value).intValue())) {
      setValue(value,row,column);
    }
  }
  
  protected void setValue(Object value,int row,int column) {
    board[row][column] = (Integer)value;
    super.setValueAt(value,row,column);
  }

  public boolean isCellEditable(int row,int column) {
    return !generated[row][column] || level == 5;
  }

  public void setGenerated(int row,int col,boolean generated) {
    this.generated[row][col] = generated;
  }

  public boolean isCellGenerated(int row,int column) {
    return generated[row][column];
  }

  public boolean accept(int r,int c,int num) {
    if(num <= 0 || num > board.length) return false;
    if(rowHas(r,num)) return false;
    if(columnHas(c,num)) return false;
    if(subBoardHas(r,c,num)) return false;
    return true;
  }

  final protected int subboardRow(int r) {
    return r-(r%subBoardSize);
  }

  final protected int subboardCol(int c) {
    return c-(c%subBoardSize);
  }

  public int getSubBoardSize() {
    return subBoardSize;
  }

  public void printBoard() {
    printBoard(System.out);
  }

  public void printBoard(PrintStream out) {
    for(int i=0;i<board.length;i++) {
      for(int j=0;j<board.length;j++)
        if(board[i][j] == null)
          out.print(" ");
        else
          out.print(board[i][j]);
      out.println();
    }
  }

  protected void setValue(int r,int c,int n) {
    board[r][c] = n;
    fireTableCellUpdated(r,c);
  }

  public void solve() {
    solve(true);
  }

  // specifically for 9x9
  public void solve(boolean fullSolution) {
    while(!isGameOver()) {
      boolean a1 = false;
      // for each number try to get all of it's occurances by row and col
      for(int i=0;i<=board.length;i++) {
        while(true) { // keep trying till can't place a number
          a1 = false;

	  // check subblocks rowwise
          for(int s=0;s<subBoardSize;s++) {
            boolean[] bexists =  { subBoardHas(s*subBoardSize,0,i),
                                  subBoardHas(s*subBoardSize,3,i),
                                  subBoardHas(s*subBoardSize,6,i) };
	    // find the single block missing the number
            int bmissing = missingIndex(bexists);
            if(bmissing < 0) continue;
            boolean[] rexists = { rowHas(s*subBoardSize,i),
                                  rowHas(s*subBoardSize+1,i),
                                  rowHas(s*subBoardSize+2,i) };
	    // find the missing row in the single block
            int rmissing = missingIndex(rexists);
            if(rmissing <0) continue;
            int cmissing;
	    // find the single column that can have this number
            cmissing = subBoardSingleFreeCellRow(s*subBoardSize,bmissing*subBoardSize,rmissing,i);
            if(cmissing >= 0) {
	      int r = s*subBoardSize+rmissing;
       	      int c = bmissing*subBoardSize+cmissing;
              if(board[r][c] == null) {
		setValue(r,c,i);
		if(!fullSolution) return;
                a1 = true;
	      }
            }
          }

	  // check subblocks columnwise
          for(int s=0;s<subBoardSize;s++) {
            boolean[] bexists =  { subBoardHas(0,s*subBoardSize,i),
                                  subBoardHas(3,s*subBoardSize,i),
                                  subBoardHas(6,s*subBoardSize,i) };
            int bmissing = missingIndex(bexists);
            if(bmissing < 0) continue;
            boolean[] cexists = { columnHas(s*subBoardSize,i),
                                  columnHas(s*subBoardSize+1,i),
                                  columnHas(s*subBoardSize+2,i) };
            int cmissing = missingIndex(cexists);
            if(cmissing <0) continue;
            int rmissing;
            rmissing = subBoardSingleFreeCellCol(bmissing*subBoardSize,s*subBoardSize,cmissing,i);
            if(rmissing >= 0) {
	      int r = bmissing*subBoardSize+rmissing;
              int c = s*subBoardSize+cmissing;
	      if(board[r][c] == null) {
		setValue(r,c,i);
		if(!fullSolution) return;
                a1 = true;
	      }
            }
          }
          if(!a1) break;          
        }
      }
      if(!a1) {
        // for each row, check if a number has a single position
	for(int i=0;i<board.length;i++) {
          for(int n=1;n<=board.length;n++) {
            if(!rowHas(i,n)) {
              int c = singleFreeCellRow(i,n);
	      if(c >= 0) {
		setValue(i,c,n);
		if(!fullSolution) return;
		a1 = true;
		break;
              }
	    }
	  }
	  if(a1) break;
	}
      }
      if(!a1) {
        // for each column, check if a number has a single position
	for(int i=0;i<board.length;i++) {
          for(int n=1;n<=board.length;n++) {
            if(!columnHas(i,n)) {
              int r = singleFreeCellCol(i,n);
	      if(r >= 0) {
		setValue(r,i,n);
		if(!fullSolution) return;
		a1 = true;
		break;
              }
	    }
	  }
	  if(a1) break;
	}
      }
      if(!a1) {
        // for each subboard, check if a number has a single position
	int[] pos = { -1 , -1 };
	for(int i=0;i<subBoardSize;i++) {
          for(int j=0;j<subBoardSize;j++) {
            for(int n=1;n<=board.length;n++) {
              if(!subBoardHas(i*subBoardSize,j*subBoardSize,n)) {
	        singleFreeCellSubBoard(i,j,n,pos);
		if(pos[0]>=0) {
		  setValue(pos[0],pos[1],n);
		if(!fullSolution) return;
		  a1 = true;
		  break;
                }
	      }
	    }
	    if(a1) break;
	  }
	  if(a1) break;
	}
      }

      int[] freeNumbers = new int[board.length];
      int[] freePositions = new int[board.length];
      if(!a1) {
	for(int i=0;i<board.length;i++) {
	  clearArray(freeNumbers);
	  int free = 0;
	  for(int l=0;l<board.length;l++) {
	    if(board[i][l] != null) continue;
	    freeNumbers[l] = possibleNumbers(i,l);
	    free++;
	  }
	  if(free > 0) {
	    sr.setBitVec(freeNumbers);
	    for(int b=2;b<free;b++) {
	      sr.resolve(b);
	      if(sr.index >= 0) {
	        setValue(i,sr.index,bitNumber(sr.bitVec[sr.index]));
		if(!fullSolution) return;
	        a1 = true;
	        break;
	      }
	    }
	    if(!a1) {
	      clearArray(freePositions);
              transpose(freeNumbers,freePositions);
	      sr.setBitVec(freePositions);
	      for(int b=2;b<free;b++) {
	        sr.resolve(b);
	        if(sr.index >= 0) {
	          setValue(i,bitNumber(sr.bitVec[sr.index])-1,sr.index+1);
		  if(!fullSolution) return;
	          a1 = true;
	          break;
	        }
	      }
	    }
	  }
	  if(a1) break;
        }
      }
      if(!a1) {
	for(int i=0;i<board.length;i++) {
	  clearArray(freeNumbers);
	  int free = 0;
	  for(int l=0;l<board.length;l++) {
	    if(board[l][i] != null) continue;
	    freeNumbers[l] = possibleNumbers(l,i);
	    free++;
	  }
	  if(free > 0) {
	    sr.setBitVec(freeNumbers);
	    for(int b=2;b<free;b++) {
	      sr.resolve(b);
	      if(sr.index >= 0) {
	        setValue(sr.index,i,bitNumber(sr.bitVec[sr.index]));
		if(!fullSolution) return;
	        a1 = true;
	        break;
	      }
	    }
	    if(!a1) {
	      clearArray(freePositions);
              transpose(freeNumbers,freePositions);
	      sr.setBitVec(freePositions);
	      for(int b=2;b<free;b++) {
	        sr.resolve(b);
	        if(sr.index >= 0) {
	          setValue(bitNumber(sr.bitVec[sr.index])-1,i,sr.index+1);
		  if(!fullSolution) return;
	          a1 = true;
	          break;
	        }
	      }
	    }
	  }
	  if(a1) break;
        }
      }
      if(!a1) {
        for(int i=0;i<subBoardSize;i++) {
          for(int j=0;j<subBoardSize;j++) {
	    clearArray(freeNumbers);
	    int free = 0;
	    for(int l=0;l<board.length;l++) {
	      int r = i*subBoardSize+l/subBoardSize;
	      int c = j*subBoardSize+l%subBoardSize;
	      if(board[r][c] != null) continue;
	      freeNumbers[l] = possibleNumbers(r,c);
	      free++;
	    }
	    if(free > 0) {
		    /*
	    if(i==1 && j==1)
	      for(int a=0;a<freeNumbers.length;a++)
	        System.out.println(Integer.toBinaryString((1<<9)+freeNumbers[a]));
	    System.out.println();
		*/
	      sr.setBitVec(freeNumbers);
	      for(int b=2;b<free;b++) {
	        sr.resolve(b);
	        if(sr.index >= 0) {
	          int r=i*subBoardSize+sr.index/subBoardSize;
		  int c=j*subBoardSize+sr.index%subBoardSize;
	          setValue(r,c,bitNumber(sr.bitVec[sr.index]));
		if(!fullSolution) return;
	          a1 = true;
	  	  break;
	        }
	      }
	      /*
	    if(i==1 && j==1)
	      for(int a=0;a<freeNumbers.length;a++)
	        System.out.println(Integer.toBinaryString((1<<9)+freeNumbers[a]));
		*/
	    if(!a1) {
	      clearArray(freePositions);
              transpose(freeNumbers,freePositions);
	      sr.setBitVec(freePositions);
	      for(int b=2;b<free;b++) {
	        sr.resolve(b);
	        if(sr.index >= 0) {
	          int r=i*subBoardSize+(sr.number-1)/subBoardSize;
		  int c=j*subBoardSize+(sr.number-1)%subBoardSize;
	          setValue(r,c,sr.index+1);
		  if(!fullSolution) return;
	          a1 = true;
	          break;
	        }
	      }
	    }
	    }
	    if(a1) break;
	  }
	  if(a1) break;
	}
      }
      if(!a1) break;
    }
  }

  public boolean isGameOver() {
    for(int i=0;i<board.length;i++) {
      for(int j=0;j<board.length;j++) {
        if(board[i][j] == null) return false;
      }
    }
    return true;
  }

  protected boolean rowHas(int row,int num) {
    for(int i=0;i<board.length;i++)
      if(board[row][i] != null && board[row][i] == num)
        return true;
    return false;
  }

  protected boolean columnHas(int col,int num) {
    for(int i=0;i<board.length;i++)
      if(board[i][col] != null && board[i][col] == num)
        return true;
    return false;
  }

  protected boolean subBoardHas(int r,int c,int num) {
    int sr = subboardRow(r);
    int sc = subboardCol(c);
    for(int i=sr;i<sr+subBoardSize;i++) {
      for(int j=sc;j<sc+subBoardSize;j++) {
        if(board[i][j] != null && board[i][j] == num) return true;
      }
    }
    return false;
  }

  protected int subBoardSingleFreeCellRow(int sr,int sc,int r,int num) {
    int free = 0;
    int index = -1;
    for(int i=sc;i<sc+subBoardSize;i++)
      if(board[sr+r][i] == null && !columnHas(i,num)) {
	free++;
	index = i-sc;
      }
    if(free == 1)
      return index;
    return -1;
  }

  protected int subBoardSingleFreeCellCol(int sr,int sc,int c,int num) {
    int free = 0;
    int index = -1;
    for(int i=sr;i<sr+subBoardSize;i++)
      if(board[i][sc+c] == null && !rowHas(i,num)) {
	free++;
	index = i-sr;
      }
    if(free == 1)
      return index;
    return -1;
  }

  // For a row, get the column that is the only possible one for the number
  protected int singleFreeCellRow(int r,int num) {
    int free = 0;
    int index = -1;
    int sr = r/subBoardSize;
    for(int i=0;i<board.length;i++) {
      int sc = i/subBoardSize;
      if(board[r][i] == null && !columnHas(i,num) && !subBoardHas(r,i,num)) {
	int c = singleColSubBoardPosition(sr,sc,num);
	if(c>=0) {
	   if((c+sc*subBoardSize) !=i) continue;
	   int count = freeCountSubBoardCol(sr,i,num);
	   if(count == 1) return i;
        }
	else {
          boolean found = false;
          for(int j=0;j<subBoardSize;j++) {
	    if(j==sr) continue;
	    int ac = singleColSubBoardPosition(j,sc,num);
	    if(ac >= 0) {
	       if(ac==i%subBoardSize) {
		 found = true;
	         break;
	       }
	    }
	  }
	  // More can be done here;
	  if(found) continue;
	}
	free++;
	index = i;
      }
    }
    if(free == 1)
      return index;
    return -1;
  }
  
  // For a column, get the row that is the only possible one for the number
  protected int singleFreeCellCol(int c,int num) {
    int free = 0;
    int index = -1;
    int sc = c/subBoardSize;
    for(int i=0;i<board.length;i++) {
      int sr = i/subBoardSize;
      if(board[i][c] == null && !rowHas(i,num) && !subBoardHas(i,c,num)) {
	int r = singleRowSubBoardPosition(sr,sc,num);
	if(r>=0) {
	   if((r+subBoardSize*sr) !=i) continue;
	   int count = freeCountSubBoardRow(i,sc,num);
	   if(count == 1) return i;
        }
	else {
          boolean found = false;
          for(int j=0;j<subBoardSize;j++) {
	    if(j==sr) continue;
	    int ar = singleRowSubBoardPosition(sr,j,num);
	    if(ar >= 0) {
	       if(ar==i%subBoardSize) {
		 found = true;
	         break;
	       }
	    }
	  }
	  if(found) continue;
	}
	free++;
	index = i;
      }
    }
    if(free == 1)
      return index;
    return -1;
  }

  // For a subblock, get the cell that is the only possible one for the number
  protected void singleFreeCellSubBoard(int sr,int sc,int num,int[] pos) {
    int free = 0;
    for(int i=sr*subBoardSize;i<(sr+1)*subBoardSize;i++)
      for(int j=sc*subBoardSize;j<(sc+1)*subBoardSize;j++) {
	if(board[i][j] == null && !rowHas(i,num) && !columnHas(j,num)) {
          pos[0] = i;
	  pos[1] = j;
	  free++;
	}
      }
    if(free != 1) {
      pos[0] = -1;
      pos[1] = -1;
    }
  }

  protected int freeCountSubBoardRow(int r,int sc,int num) {
    int free = 0;
    for(int i=sc*subBoardSize;i<(sc+1)*subBoardSize;i++)
      if(board[r][i] == null && !columnHas(i,num))
	free++;
    return free;
  }

  protected int freeCountSubBoardCol(int sr,int c,int num) {
    int free = 0;
    for(int i=sr*subBoardSize;i<(sr+1)*subBoardSize;i++)
      if(board[i][c] == null && !rowHas(i,num))
	free++;
    return free;
  }

  protected int possibleNumbers(int r,int c) {
    int num = 0;
    int sr = r/subBoardSize;
    int sc = c/subBoardSize;
    for(int i=1;i<=board.length;i++)
      if(!(subBoardHas(r,c,i) || rowHas(r,i) || columnHas(c,i))) {
	boolean found = false;
	for(int j=0;j<subBoardSize;j++) {
          if(j==sc) continue;
	  if(singleRowSubBoardPosition(sr,j,i) == (r-sr*subBoardSize)) {
            found = true;
	    break;
	  }
	}
	if(!found) {
	  for(int j=0;j<subBoardSize;j++) {
            if(j==sr) continue;
	    if(singleColSubBoardPosition(j,sc,i) == (c-sc*subBoardSize)) {
              found = true;
	      break;
	    }
	  }
	}
	if(!found) {
          int row = possibleRow(sr,sc,i);
	  if(row>=0 && (sr*subBoardSize+row) != r)
	    continue;
	}
	if(!found) {
          int col = possibleCol(sr,sc,i);
	  if(col>=0 && (sc*subBoardSize+col) != c)
	    continue;
	}
	if(!found)
	  num |= bits[i-1];
      }
    return num;
  }

  protected int possibleRows(int sr,int sc,int num) {
    int rows = 0;
    for(int i=0;i<subBoardSize;i++) {
      for(int j=0;j<subBoardSize;j++) {
	int r = sr*subBoardSize+i;
	int c = sc*subBoardSize+j;
	if(board[r][c] == null) {
          if(!columnHas(c,num)) {
	    if(!rowHas(r,num)) {
	      rows |= bits[i];
	      j = subBoardSize;
	    }
	  }
	}
	else if(board[r][c] == num)
          return bits[i];
      }
    }
    return rows;
  }

  protected int possibleCols(int sr,int sc,int num) {
    int cols = 0;
    for(int i=0;i<subBoardSize;i++) {
      for(int j=0;j<subBoardSize;j++) {
	int c = sc*subBoardSize+i;
	int r = sr*subBoardSize+j;
	if(board[r][c] == null) {
	  if(!rowHas(r,num)) {
	    if(!columnHas(c,num)) {
	      cols |= bits[i];
	      j = subBoardSize;
	    }
  	  }
	}
	else if(board[r][c] == num)
          return bits[i];
      }
    }
    return cols;
  }

  protected int possibleRow(int sr,int sc,int num) {
    int rows = possibleRows(sr,sc,num);
    int row = bitNumber(rows)-1;
    if(row >= 0) return row;
    rows = 0;
    for(int i=0;i<subBoardSize;i++) 
      if(i!=sc)
        rows |= possibleRows(sr,i,num);
    row = -1;
    for(int i=0;i<subBoardSize;i++)
      if((rows&bits[i])==0)
        if(row<0)
	  row = i;
	else
	  return -1;
    return row;
  }

  protected int possibleCol(int sr,int sc,int num) {
    int cols = possibleCols(sr,sc,num);
    int col = bitNumber(cols)-1;
    if(col >= 0) return col;
    cols = 0;
    for(int i=0;i<subBoardSize;i++)
      if(i!=sr)
        cols |= possibleCols(i,sc,num);
    col = -1;
    for(int i=0;i<subBoardSize;i++)
      if((cols&bits[i])==0)
        if(col<0) 
	  col = i;
	else
	  return -1;
    return col;
  }

  protected int singleRowSubBoardPosition(int sr,int sc,int num) {
    int map = 7;
    for(int i=0;i<subBoardSize;i++) {
      boolean found = false;
      int pos = 0;
      int count = 0;
      for(int j=sr*subBoardSize;j<(sr+1)*subBoardSize;j++) {
	for(int k=i*subBoardSize;k<(i+1)*subBoardSize;k++) {
          if(board[j][k] != null) {
            if(board[j][k] == num) {
	      pos = bits[(j-sr*subBoardSize)];
	      count = 1;
	      found = true;
	      break;
	    }
	  }
	  else {
	    if(!(columnHas(k,num) || rowHas(j,num))) {
	      pos |= bits[(j-sr*subBoardSize)];
	      count++;
	      break;
	    }
	  }
	}
	if(found) break;
      }
      if(count == 1 && i == sc) {
        map = pos;
	break;
      }
      if(found || count == 1) {
        map ^= pos;
      }
    }
    for(int i=0;i<subBoardSize;i++)
      if(bits[i] == map) 
	return i;
    return -1;
  }

  protected int singleColSubBoardPosition(int sr,int sc,int num) {
    int map = 7;
    for(int i=0;i<subBoardSize;i++) {
      boolean found = false;
      int pos = 0;
      int count = 0;
      for(int j=sc*subBoardSize;j<(sc+1)*subBoardSize;j++) {
	for(int k=i*subBoardSize;k<(i+1)*subBoardSize;k++) {
          if(board[k][j] != null) {
            if(board[k][j] == num) {
	      pos = bits[(j-sc*subBoardSize)];
	      count = 1;
	      found = true;
	      break;
	    }
	  }
	  else {
	    if(!(columnHas(j,num) || rowHas(k,num))) {
	      pos |= bits[(j-sc*subBoardSize)];
	      count++;
	      break;
	    }
	  }
	}
	if(found) break;
      }
      if(count == 1 && i == sr) {
        map = pos;
	break;
      }
      if(found || count == 1) {
        map ^= pos;
      }
    }
    for(int i=0;i<subBoardSize;i++)
      if(bits[i] == map) 
	return i;
    return -1;
  }

  protected Integer[][] board;
  protected boolean[][] generated;
  protected int subBoardSize;
  protected int level;

  protected SetResolver sr = new SetResolver();

  protected static int[] bits = new int[9];

  static {
    bits[0] = 1;
    for(int i=1;i<bits.length;i++)
      bits[i] = bits[i-1]*2;
  }

  // Given a 2^x number, return x+1, otherwise -1
  static int bitNumber(int n) {
    for(int i=0;i<bits.length;i++)
      if(bits[i] == n) return i+1;
    return -1;
  }

  static void clearArray(int[] array) {
    for(int i=0;i<array.length;i++)
      array[i] = 0;
  }

  static private int missingIndex(boolean[] exists) {
    int result = -2;
    if(exists[0] && exists[1] && !exists[2]) 
      result = 2;
    else if(!exists[0] && exists[1] && exists[2])
      result = 0;
    else if(exists[0] && !exists[1] && exists[2])
      result = 1;
    else if(exists[0] && exists[1] && exists[2]) 
      result = -1;
    return result;
  }

  // subtract any bit in b that's also present in a
  static int bitwisesubtract(int a,int b) {
    int n = 1;
    while(n>0 && b>0 && a>0) {
      if((b&n)>0 && (a&n)>0)
	a -= n;
      n <<=1;
    }
    return a;
  }

  // count the number of bits in i
  static int bitCount(int i) {
    int c = 0;
    for(;i>0;i>>=1)
      if((i&1)==1) c++;
    return c;
  }

  // transpose bit matrix of a into bit matrix of b
  static void transpose(int[] a,int[] b) {
    for(int i=0;i<a.length;i++) {
      if(a[i] == 0) continue;
      for(int j=0;j<a.length;j++)
	if((a[i]&bits[j])>0)
	  b[j] |= bits[i];
    }
  }

  public boolean newBoard() {
    SudokuModel sm = generateSudoku(level);
    if(sm != null) {
      this.board = sm.board;
      this.generated = sm.generated;
      fireTableDataChanged();
      return true;
    }
    return false;
  }

  public void reset() {
    for(int i=0;i<board.length;i++)
      for(int j=0;j<board.length;j++)
        if(!generated[i][j])
          setValueAt(null,i,j);
    fireTableDataChanged();
  }

  public void save() {
    PrintStream out = null;
    try {
      out = new PrintStream(new FileOutputStream("sudoku.txt"));
      printBoard(out);
    }
    catch(Exception ex) {
    }
    finally {
      if(out != null) try { out.close(); } catch(Exception ex) {};
    }
  }
}

/*
 *  Given an array of bitvectors (ints), can identify the list
 *  of items that form a closed group and remove the corresponding
 *  bits from the remaining items
 */
class SetResolver implements NCR.CombinationListener {

  public void setBitVec(int[] bitVec) {
    this.bitVec = bitVec;
    index = -1;
    number =-1;
  }

  public void resolve(int r) {
    index = -1;
    if(!findSingleBitIndex()) {
      NCR.ncr(bitVec.length,r,this);
    }
  }

  private boolean findSingleBitIndex() {
    for(int i=0;i<bitVec.length;i++) {
      if(SudokuModel.bitCount(bitVec[i]) == 1) {
	index = i;
	number = bitVec[i];
	return true;
      }
    }
    return false;
  }

  public boolean combination(int[] combination) {
    int bits = 0;
    for(int i=0;i<combination.length;i++) {
      int b = bitVec[combination[i]-1];
      if(b == 0) return true;
      bits |= b;
    }
    if(SudokuModel.bitCount(bits) == combination.length) {
      for(int i=0;i<bitVec.length;i++) {
	if(bitVec[i] > 0)
          bitVec[i] = SudokuModel.bitwisesubtract(bitVec[i],bits);
      }
      findSingleBitIndex();
      return false;
    }
    return true;
  }

  protected int[] bitVec;
  protected int index;
  protected int number;
}
